==> etterna-0.68.0.tar.gz: etterna-0.68.0/src/Etterna/Singletons/PrefsManager.h <==
#ifndef PREFSMANAGER_H
#define PREFSMANAGER_H

#include "Etterna/Models/Misc/GameConstantsAndTypes.h"
#include "Etterna/Models/Misc/Preference.h"

class IniFile;

void
ValidateDisplayAspectRatio(float& val);

enum MusicWheelUsesSections
{
	MusicWheelUsesSections_NEVER,
	MusicWheelUsesSections_ALWAYS,
	MusicWheelUsesSections_ABC_ONLY,
	NUM_MusicWheelUsesSections,
	MusicWheelUsesSections_Invalid
};
/** @brief The options for allowing the W1 timing. */
enum AllowW1
{
	ALLOW_W1_NEVER,		 /**< The W1 timing is not used. */
	ALLOW_W1_EVERYWHERE, /**< The W1 timing is used for all modes. */
	NUM_AllowW1,
	AllowW1_Invalid
};
enum Maybe
{
	Maybe_ASK,
	Maybe_NO,
	Maybe_YES,
	NUM_Maybe,
	Maybe_Invalid
};
enum GetRankingName
{
	RANKING_OFF,
	RANKING_ON,
	RANKING_LIST,
	NUM_GetRankingName,
	GetRankingName_Invalid
};
enum RandomBackgroundMode
{
	BGMODE_OFF,
	BGMODE_ANIMATIONS,
	BGMODE_RANDOMMOVIES,
	NUM_RandomBackgroundMode,
	RandomBackgroundMode_Invalid
};
enum ImageCacheMode
{
	IMGCACHE_OFF,
	IMGCACHE_LOW_RES_PRELOAD,		 // preload low-res on start
	IMGCACHE_LOW_RES_LOAD_ON_DEMAND, // preload low-res on screen load
	IMGCACHE_FULL,
	NUM_ImageCacheMode,
	ImageCacheMode_Invalid
};
enum HighResolutionTextures
{
	HighResolutionTextures_Auto,
	HighResolutionTextures_ForceOff,
	HighResolutionTextures_ForceOn,
	NUM_HighResolutionTextures,
	HighResolutionTextures_Invalid,
};
enum AttractSoundFrequency
{
	ASF_NEVER,
	ASF_EVERY_TIME,
	ASF_EVERY_2_TIMES,
	ASF_EVERY_3_TIMES,
	ASF_EVERY_4_TIMES,
	ASF_EVERY_5_TIMES,
	NUM_AttractSoundFrequency,
	AttractSoundFrequency_Invalid
};
enum BackgroundFitMode
{
	BFM_CoverDistort,
	BFM_CoverPreserve,
	BFM_FitInside,
	BFM_FitInsideAvoidLetter,
	BFM_FitInsideAvoidPillar,
	NUM_BackgroundFitMode,
	BackgroundFitMode_Invalid
};

/** @brief Holds user-chosen preferences that are saved between sessions. */
class PrefsManager
{
  public:
	PrefsManager();
	~PrefsManager();

	void Init();

	void SetCurrentGame(const RString& sGame);
	RString GetCurrentGame() { return m_sCurrentGame; }

  protected:
	Preference<RString> m_sCurrentGame;

  public:
	// Game-specific prefs.  Copy these off and save them every time the game
	// changes
	Preference<RString> m_sAnnouncer;
	Preference<RString> m_sTheme;
	Preference<RString> m_sDefaultModifiers;

  protected:
	void StoreGamePrefs();
	void RestoreGamePrefs();
	struct GamePrefs
	{
		// See GamePrefs::GamePrefs in PrefsManager.cpp for some default
		// settings
		GamePrefs();

		RString m_sAnnouncer;
		RString m_sTheme;
		RString m_sDefaultModifiers;
	};
	map<RString, GamePrefs> m_mapGameNameToGamePrefs;

  public:
	Preference<bool> m_bWindowed;
	Preference<RString> m_sDisplayId;
	Preference<int> m_iDisplayWidth;
	Preference<int> m_iDisplayHeight;
	Preference<float> m_fDisplayAspectRatio;
	Preference<int> m_iDisplayColorDepth;
	Preference<int> m_iTextureColorDepth;
	Preference<int> m_iMovieColorDepth;
	Preference<bool> m_bStretchBackgrounds;
	Preference<BackgroundFitMode> m_BGFitMode;
	Preference<HighResolutionTextures> m_HighResolutionTextures;
	Preference<int> m_iMaxTextureResolution;
	Preference<int> m_iRefreshRate;
	Preference<bool> m_bAllowMultitexture;
	Preference<float> m_bAllowedLag;
	Preference<bool> m_bFullscreenIsBorderlessWindow;
	Preference<bool> m_bShowStats;
	Preference<bool> m_bShowSkips;
	Preference<bool> m_bShowMouseCursor;
	Preference<bool> m_bVsync;
	Preference<bool> m_FastNoteRendering;
	Preference<bool> m_bInterlaced;
	Preference<bool> m_bPAL;
	Preference<bool> m_bDelayedTextureDelete;
	Preference<bool> m_bDelayedModelDelete;
	Preference<ImageCacheMode> m_ImageCache;
	Preference<bool> m_bFastLoad;
	Preference<bool> m_bBlindlyTrustCache;
	Preference<bool> m_bShrinkSongCache;
	Preference<RString> m_NeverCacheList;

	Preference<bool> m_bOnlyDedicatedMenuButtons;
	Preference<bool> m_bMenuTimer;

	Preference<float> m_fLifeDifficultyScale;

	// Whoever added these: Please add a comment saying what they do. -Chris
	Preference<int> m_iRegenComboAfterMiss;	// combo that must be met after a
											   // Miss to regen life
	Preference<int> m_iMaxRegenComboAfterMiss; // caps RegenComboAfterMiss if
											   // multiple Misses occur in rapid
											   // succession
	Preference<bool> m_bDelayedBack;
	Preference<bool> m_AllowHoldForOptions;
	Preference<bool> m_bShowInstructions; // how to play a mode
	Preference<bool> m_bShowNativeLanguage;
	Preference<bool> m_bFullTapExplosions;
	Preference<bool> m_bNoGlow;
	Preference<bool> m_bReplaysUseScoreMods;
	Preference<int> m_iArcadeOptionsNavigation;
	Preference<bool> m_ThreeKeyNavigation;
	Preference<MusicWheelUsesSections> m_MusicWheelUsesSections;
	Preference<int> m_iMusicWheelSwitchSpeed;
	Preference<bool> m_bSortBySSRNorm;
	Preference<AllowW1>
	  m_AllowW1; // this should almost always be on, given use cases. -aj
	Preference<bool> m_bEventMode;
	Preference<TapNoteScore> m_MinTNSToHideNotes;

	Preference<Maybe> m_ShowSongOptions;
	Preference<float> m_fMinPercentToSaveScores;
	Preference<float> m_fGlobalOffsetSeconds;
	Preference<RString> m_sLanguage;
	Preference<int> m_iCenterImageTranslateX;
	Preference<int> m_iCenterImageTranslateY;
	Preference<int> m_fCenterImageAddWidth;
	Preference<int> m_fCenterImageAddHeight;
	Preference<bool> m_bCelShadeModels;
	Preference<bool> m_bPreferredSortUsesGroups;
	Preference<bool> EnablePitchRates;
	Preference<bool> LiftsOnOsuHolds;
	Preference<bool> m_bEasterEggs;
	Preference<bool> m_bUseMidGrades;

	// Number of seconds it takes for a button on the controller to release
	// after pressed.
	Preference<float> m_fPadStickSeconds;

	// Useful for non 4:3 displays and resolutions < 640x480 where texels don't
	// map directly to pixels.
	Preference<bool> m_bForceMipMaps;
	Preference<bool> m_bTrilinearFiltering; // has no effect without mipmaps on
	Preference<bool> m_bAnisotropicFiltering; // has no effect without mipmaps
											  // on.  Not mutually exclusive
											  // with trilinear.

	// If true, then signatures created when writing profile data and verified
	// when reading profile data. Leave this false if you want to use a profile
	// on different machines that don't have the same key, or else the
	// profile's data will be discarded.
	Preference<bool> m_bSignProfileData;

	Preference<RString> m_sAdditionalSongFolders;
	Preference<RString> m_sAdditionalFolders;

	// failsafe
	Preference<RString> m_sDefaultTheme;

	Preference<RString> m_sLastSeenVideoDriver;
	Preference<RString> m_sVideoRenderers; // StepMania.cpp sets these on first
										   // run based on the card
	Preference<bool> m_bSmoothLines;
	Preference<int> m_iSoundWriteAhead;
	Preference<RString> m_iSoundDevice;
	Preference<int> m_iSoundPreferredSampleRate;
	Preference<bool> m_bAllowUnacceleratedRenderer;
	Preference<bool> m_bThreadedInput;
	Preference<bool> m_bThreadedMovieDecode;
	Preference<RString> m_sTestInitialScreen;
	Preference<bool> m_MuteActions;
	Preference<int> ThreadsToUse;

	// Debug:
	Preference<bool> m_bLogToDisk;
	Preference<bool> m_bForceLogFlush;
	Preference<bool> m_bShowLogOutput;
	Preference<bool> m_bLogSkips;
	Preference<bool> m_bLogCheckpoints;
	Preference<bool> m_bShowLoadingWindow;
	Preference<bool> m_bPseudoLocalize;
	Preference<bool> m_show_theme_errors;

	Preference<int> m_verbose_log; // levels 0, 1, and 2 where higher numbers
								   // means more logging

	Preference<bool>
	  m_bEnableScoreboard; // Alows disabling of scoreboard in network play

	void ReadPrefsFromIni(const IniFile& ini,
						  const RString& sSection,
						  bool bIsStatic);
	void ReadGamePrefsFromIni(const RString& sIni);
	void ReadDefaultsFromIni(const IniFile& ini, const RString& sSection);
	void SavePrefsToIni(IniFile& ini);

	void ReadPrefsFromDisk();
	void SavePrefsToDisk();

	void ResetToFactoryDefaults();

	RString GetPreferencesSection() const;

	// Lua
	void PushSelf(lua_State* L);

  protected:
	void ReadPrefsFromFile(const RString& sIni,
						   const RString& sSection,
						   bool bIsStatic);
	void ReadDefaultsFromFile(const RString& sIni, const RString& sSection);
};

/* This is global, because it can be accessed by crash handlers and error
 * handlers that are run after PREFSMAN shuts down (and probably don't want to
 * deref that pointer anyway). */
extern bool g_bAutoRestart;

extern PrefsManager*
  PREFSMAN; // global and accessible from anywhere in our program

#endif

==> etterna-0.69.1.tar.gz: etterna-0.69.1/src/Etterna/Singletons/PrefsManager.h <==
#ifndef PREFSMANAGER_H
#define PREFSMANAGER_H

#include "Etterna/Models/Misc/GameConstantsAndTypes.h"
#include "Etterna/Models/Misc/Preference.h"

class IniFile;

void
ValidateDisplayAspectRatio(float& val);

enum MusicWheelUsesSections
{
	MusicWheelUsesSections_NEVER,
	MusicWheelUsesSections_ALWAYS,
	MusicWheelUsesSections_ABC_ONLY,
	NUM_MusicWheelUsesSections,
	MusicWheelUsesSections_Invalid
};
/** @brief The options for allowing the W1 timing. */
enum AllowW1
{
	ALLOW_W1_NEVER,		 /**< The W1 timing is not used. */
	ALLOW_W1_EVERYWHERE, /**< The W1 timing is used for all modes. */
	NUM_AllowW1,
	AllowW1_Invalid
};
enum Maybe
{
	Maybe_ASK,
	Maybe_NO,
	Maybe_YES,
	NUM_Maybe,
	Maybe_Invalid
};
enum GetRankingName
{
	RANKING_OFF,
	RANKING_ON,
	RANKING_LIST,
	NUM_GetRankingName,
	GetRankingName_Invalid
};
enum RandomBackgroundMode
{
	BGMODE_OFF,
	BGMODE_ANIMATIONS,
	BGMODE_RANDOMMOVIES,
	NUM_RandomBackgroundMode,
	RandomBackgroundMode_Invalid
};
enum ImageCacheMode
{
	IMGCACHE_OFF,
	IMGCACHE_LOW_RES_PRELOAD,		 // preload low-res on start
	IMGCACHE_LOW_RES_LOAD_ON_DEMAND, // preload low-res on screen load
	IMGCACHE_FULL,
	NUM_ImageCacheMode,
	ImageCacheMode_Invalid
};
enum HighResolutionTextures
{
	HighResolutionTextures_Auto,
	HighResolutionTextures_ForceOff,
	HighResolutionTextures_ForceOn,
	NUM_HighResolutionTextures,
	HighResolutionTextures_Invalid,
};
enum AttractSoundFrequency
{
	ASF_NEVER,
	ASF_EVERY_TIME,
	ASF_EVERY_2_TIMES,
	ASF_EVERY_3_TIMES,
	ASF_EVERY_4_TIMES,
	ASF_EVERY_5_TIMES,
	NUM_AttractSoundFrequency,
	AttractSoundFrequency_Invalid
};
enum BackgroundFitMode
{
	BFM_CoverDistort,
	BFM_CoverPreserve,
	BFM_FitInside,
	BFM_FitInsideAvoidLetter,
	BFM_FitInsideAvoidPillar,
	NUM_BackgroundFitMode,
	BackgroundFitMode_Invalid
};

/** @brief Holds user-chosen preferences that are saved between sessions. */
class PrefsManager
{
  public:
	PrefsManager();
	~PrefsManager();

	void Init();

	void SetCurrentGame(const RString& sGame);
	RString GetCurrentGame() { return m_sCurrentGame; }

  protected:
	Preference<RString> m_sCurrentGame;

  public:
	// Game-specific prefs.  Copy these off and save them every time the game
	// changes
	Preference<RString> m_sAnnouncer;
	Preference<RString> m_sTheme;
	Preference<RString> m_sDefaultModifiers;

  protected:
	void StoreGamePrefs();
	void RestoreGamePrefs();
	struct GamePrefs
	{
		// See GamePrefs::GamePrefs in PrefsManager.cpp for some default
		// settings
		GamePrefs();

		RString m_sAnnouncer;
		RString m_sTheme;
		RString m_sDefaultModifiers;
	};
	map<RString, GamePrefs> m_mapGameNameToGamePrefs;

  public:
	Preference<bool> m_bWindowed;
	Preference<RString> m_sDisplayId;
	Preference<int> m_iDisplayWidth;
	Preference<int> m_iDisplayHeight;
	Preference<float> m_fDisplayAspectRatio;
	Preference<int> m_iDisplayColorDepth;
	Preference<int> m_iTextureColorDepth;
	Preference<int> m_iMovieColorDepth;
	Preference<bool> m_bStretchBackgrounds;
	Preference<BackgroundFitMode> m_BGFitMode;
	Preference<HighResolutionTextures> m_HighResolutionTextures;
	Preference<int> m_iMaxTextureResolution;
	Preference<int> m_iRefreshRate;
	Preference<bool> m_bAllowMultitexture;
	Preference<float> m_bAllowedLag;
	Preference<bool> m_bFullscreenIsBorderlessWindow;
	Preference<bool> m_bShowStats;
	Preference<bool> m_bShowSkips;
	Preference<bool> m_bShowMouseCursor;
	Preference<bool> m_bVsync;
	Preference<bool> m_FastNoteRendering;
	Preference<bool> m_bInterlaced;
	Preference<bool> m_bPAL;
	Preference<bool> m_bDelayedTextureDelete;
	Preference<bool> m_bDelayedModelDelete;
	Preference<ImageCacheMode> m_ImageCache;
	Preference<bool> m_bFastLoad;
	Preference<bool> m_bBlindlyTrustCache;
	Preference<bool> m_bShrinkSongCache;
	Preference<RString> m_NeverCacheList;

	Preference<bool> m_bOnlyDedicatedMenuButtons;
	Preference<bool> m_bMenuTimer;

	Preference<float> m_fLifeDifficultyScale;

	// Whoever added these: Please add a comment saying what they do. -Chris
	Preference<int> m_iRegenComboAfterMiss;	   // combo that must be met after a
											   // Miss to regen life
	Preference<int> m_iMaxRegenComboAfterMiss; // caps RegenComboAfterMiss if
											   // multiple Misses occur in rapid
											   // succession
	Preference<bool> m_bDelayedBack;
	Preference<bool> m_AllowHoldForOptions;
	Preference<bool> m_bShowInstructions;
	Preference<bool> m_bShowNativeLanguage;
	Preference<bool> m_bFullTapExplosions;
	Preference<bool> m_bNoGlow;
	Preference<bool> m_bReplaysUseScoreMods;
	Preference<int> m_iArcadeOptionsNavigation;
	Preference<bool> m_ThreeKeyNavigation;
	Preference<MusicWheelUsesSections> m_MusicWheelUsesSections;
	Preference<int> m_iMusicWheelSwitchSpeed;
	Preference<bool> m_bSortBySSRNorm;
	Preference<AllowW1>
	  m_AllowW1; // this should almost always be on, given use cases. -aj
	Preference<bool> m_bEventMode;
	Preference<TapNoteScore> m_MinTNSToHideNotes;

	Preference<Maybe> m_ShowSongOptions;
	Preference<float> m_fMinPercentToSaveScores;
	Preference<float> m_fGlobalOffsetSeconds;
	Preference<RString> m_sLanguage;
	Preference<int> m_iCenterImageTranslateX;
	Preference<int> m_iCenterImageTranslateY;
	Preference<int> m_fCenterImageAddWidth;
	Preference<int> m_fCenterImageAddHeight;
	Preference<bool> m_bCelShadeModels;
	Preference<bool> m_bPreferredSortUsesGroups;
	Preference<bool> EnablePitchRates;
	Preference<bool> LiftsOnOsuHolds;
	Preference<bool> m_bEasterEggs;
	Preference<bool> m_bUseMidGrades;

	// Number of seconds it takes for a button on the controller to release
	// after pressed.
	Preference<float> m_fPadStickSeconds;

	// Useful for non 4:3 displays and resolutions < 640x480 where texels don't
	// map directly to pixels.
	Preference<bool> m_bForceMipMaps;
	Preference<bool> m_bTrilinearFiltering; // has no effect without mipmaps on
	Preference<bool> m_bAnisotropicFiltering; // has no effect without mipmaps
											  // on.  Not mutually exclusive
											  // with trilinear.

	// If true, then signatures created when writing profile data and verified
	// when reading profile data. Leave this false if you want to use a profile
	// on different machines that don't have the same key, or else the
	// profile's data will be discarded.
	Preference<bool> m_bSignProfileData;

	Preference<RString> m_sAdditionalSongFolders;
	Preference<RString> m_sAdditionalFolders;

	// failsafe
	Preference<RString> m_sDefaultTheme;

	Preference<RString> m_sLastSeenVideoDriver;
	Preference<RString> m_sVideoRenderers; // StepMania.cpp sets these on first
										   // run based on the card
	Preference<bool> m_bSmoothLines;
	Preference<int> m_iSoundWriteAhead;
	Preference<RString> m_iSoundDevice;
	Preference<int> m_iSoundPreferredSampleRate;
	Preference<bool> m_bAllowUnacceleratedRenderer;
	Preference<bool> m_bThreadedInput;
	Preference<bool> m_bThreadedMovieDecode;
	Preference<RString> m_sTestInitialScreen;
	Preference<bool> m_MuteActions;
	Preference<int> ThreadsToUse;

	// Debug:
	Preference<bool> m_bLogToDisk;
	Preference<bool> m_bForceLogFlush;
	Preference<bool> m_bShowLogOutput;
	Preference<bool> m_bLogSkips;
	Preference<bool> m_bLogCheckpoints;
	Preference<bool> m_bShowLoadingWindow;
	Preference<bool> m_bPseudoLocalize;
	Preference<bool> m_show_theme_errors;

	// levels 0, 1, and 2 where higher numbers
	// means more logging
	Preference<int> m_verbose_log;

	Preference<bool>
	  m_bEnableScoreboard; // Alows disabling of scoreboard in network play

	void ReadPrefsFromIni(const IniFile& ini,
						  const RString& sSection,
						  bool bIsStatic);
	void ReadGamePrefsFromIni(const RString& sIni);
	void ReadDefaultsFromIni(const IniFile& ini, const RString& sSection);
	void SavePrefsToIni(IniFile& ini);

	void ReadPrefsFromDisk();
	void SavePrefsToDisk();

	void ResetToFactoryDefaults();

	RString GetPreferencesSection() const;

	// Lua
	void PushSelf(lua_State* L);

  protected:
	void ReadPrefsFromFile(const RString& sIni,
						   const RString& sSection,
						   bool bIsStatic);
	void ReadDefaultsFromFile(const RString& sIni, const RString& sSection);
};

/* This is global, because it can be accessed by crash handlers and error
 * handlers that are run after PREFSMAN shuts down (and probably don't want to
 * deref that pointer anyway). */
extern bool g_bAutoRestart;

extern PrefsManager*
  PREFSMAN; // global and accessible from anywhere in our program

#endif
